#ifndef _ARCH_ADDRS_H
#define _ARCH_ADDRS_H

#define KERNEL_START  0xffffffff80000000
#define PHYSMEM_START 0xffff800000000000

#ifdef __cplusplus
#include <kern/types.h>

// GCC can't currently handle full 64-bit code relocations
// on x64, so the code is mapped separately from the
// all-of-RAM mapping.  This means that there are two different
// kernel-virtual addresses for some physical addresses.

namespace Arch {
	static inline ulong kvirt_to_phys(void *addr)
	{
		ulong ret = reinterpret_cast<ulong>(addr);
	
		if (ret > KERNEL_START)
			return ret - KERNEL_START;
		else
			return ret - PHYSMEM_START;
	}

	static inline void *phys_to_kvirt(ulong addr)
	{
		return reinterpret_cast<void *>(addr + PHYSMEM_START);
	}
	
	namespace Priv {
		static const ulong max_ktext_map = 4 * 1024 * 1024;
	
		static inline void *phys_to_ktext(ulong addr)
		{
			return reinterpret_cast<void *>(addr + KERNEL_START);
		}
	}
};

#endif
#endif
