#ifndef _ARCH_PAGETABLE_H
#define _ARCH_PAGETABLE_H

#include <lowlevel/misc.h>
#include <lowlevel/atomic.h>

namespace Arch {
	union PTE {
		typedef ulong Word;
	
		struct {
			Word Valid:1;
			Word Writeable:1;
			Word User:1;
			Word WriteThrough:1;
			Word CacheDisable:1;
			Word Accessed:1;
			Word Dirty:1;
			Word PageAttrTable:1;
			Word Global:1;
			Word Avail1:3;
			Word Addr:40;
			Word Avail2:11;
			Word NoExec:1;
		};
		
		Word raw;

		PTE(Word init) : raw(init)
		{
		}
		
		PTE() : raw(0)
		{
		}

		operator Word()
		{
			return raw;
		}

		static Word addr_to_offset(Word addr, int shift)
		{
			int pages_per_table = page_size / sizeof(Word);
			return (addr >> shift) & (pages_per_table - 1);
		}
	
		Word pte_to_addr()
		{
			return raw & 0x000ffffffffff000;
		}
	
		static PTE addr_to_pte(Word phys)
		{
			return phys & 0x000ffffffffff000;
		}
		
		static void flags_to_pte(Mem::PTEFlags flagsin,
		                         Mem::PTEFlags maskin,
		                         PTE &flagsout,
		                         PTE &maskout)
		{
			maskout = ~0UL;
			flagsout = 0;
			
			if (maskin.Valid) {
				maskout.Valid = 0;
				flagsout.Valid = flagsin.Valid;
			}
			
			if (maskin.Writeable) {
				maskout.Writeable = 0;
				flagsout.Writeable = flagsin.Writeable;
			}
			
			// FIXME: check for and enable NX support
			if (0 && maskin.Executable) {
				maskout.NoExec = 0;
				flagsout.NoExec = !flagsin.Executable;
			}
			
			if (maskin.User) {
				maskout.User = 0;
				flagsout.User = flagsin.User;
			}
		}
	
		Mem::PTEFlags pte_to_flags()
		{
			Mem::PTEFlags ret = 0;
			
			ret.Valid = Valid;
			ret.User = User;
			
			if (Valid) {
				ret.Readable = 1;
				ret.Writeable = Writeable;
				ret.Executable = !NoExec;
			}
			
			return ret;
		}
	
		void set_pte(PTE *table, uint offset)
		{
			table[offset] = raw;
		}
		
		PTE xchg_pte(PTE *table, uint offset)
		{
			return ll_xchg_long(reinterpret_cast<Word *>(&table[offset]), raw);
		}
		
		bool valid_pte()
		{
			return Valid;
		}
		
		bool dirty_pte()
		{
			return Dirty;
		}
		
		static void invalidate_tlb_entry(Word addr)
		{
			ll_invalidate_tlb_entry(addr);
		}
		
		enum {
			page_size = Arch::page_size,
			page_shift = Arch::page_shift,
	
			pages_per_table = page_size / sizeof(Word),
			num_levels = 4,
			shift_per_level = 9,
	
			// All "upper" addresses belong to the kernel.
			
			kmap_start = 256,
			kmap_end = 511
		};
	};
}

#endif
