// lib/libc.cc -- Standard C-library functions
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors or contributors be held liable for any damages
// arising from the use of this software.
// 
// Permission is hereby granted to everyone, free of charge, to use, copy,
// modify, prepare derivative works of, publish, distribute, perform,
// sublicense, and/or sell copies of the Software, provided that the above
// copyright notice and disclaimer of warranty be included in all copies or
// substantial portions of this software.


#include <kern/types.h>
#include <kern/libc.h>
#include <kern/console.h>

#include <stdarg.h>
#include <limits.h>

void bzero(void *b, size_t len)
{
	char *c = static_cast<char *>(b);
	
	while (len--)
		*c++ = 0;
}

#include <kern/pagealloc.h>

// Temporary hack until slab allocator is added

void *malloc(size_t len)
{
	assert(len <= Arch::page_size - sizeof(size_t));

	len = (len + sizeof(size_t) + Arch::page_size - 1) / Arch::page_size;
	Mem::Page *page = Mem::PageAlloc::alloc(len);
	
	size_t *ptr = (size_t *)Mem::page_to_kvirt(page);
	*ptr = len;
	
	return ptr + 1;
}

void free(void *addr)
{
	if (addr) {
		size_t *ptr = (size_t *)addr - 1;
		size_t len = *ptr;
		Mem::Page *page = Mem::kvirt_to_page(addr);
		Mem::PageAlloc::free(page, len);
	}
}

void *operator new(size_t len)
{
	return malloc(len);
}

void *operator new[](size_t len)
{
	return malloc(len);
}

void operator delete(void *addr)
{
	free(addr);
}

void operator delete[](void *addr)
{
	free(addr);
}

extern "C" void __cxa_pure_virtual()
{
	BUG();
}

void abort()
{
	in_fault++;
	printf("abort() called in kernel\n");
	__builtin_trap();
}
