// tests/mutex.cc
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors or contributors be held liable for any damages
// arising from the use of this software.
// 
// Permission is hereby granted to everyone, free of charge, to use, copy,
// modify, prepare derivative works of, publish, distribute, perform,
// sublicense, and/or sell copies of the Software, provided that the above
// copyright notice and disclaimer of warranty be included in all copies or
// substantial portions of this software.

#include <kern/types.h>
#include <kern/libc.h>
#include <kern/time.h>
#include <kern/thread.h>
#include <util/lock.h>

Lock::Lock testlock;

void threadb(void *arg)
{
	testlock.lock();
	
	while (true) {
		Time::Time now;
		Time::monotonic_clock.get_time(&now);
		
		while (true) {
			printf("B");
		
			Time::Time now2;
			Time::monotonic_clock.get_time(&now2);

			if (now.seconds != now2.seconds)
				break;
		}
		
		testlock.unlock();
		testlock.lock();
	}
}

void threada(void *arg)
{
	Threads::sched.new_thread(threadb, NULL, "thread b")->wake();

	testlock.lock();
	
	while (true) {
		Time::Time now;
		Time::monotonic_clock.get_time(&now);
		
		while (true) {
			printf("A");
		
			Time::Time now2;
			Time::monotonic_clock.get_time(&now2);

			if (now.seconds != now2.seconds)
				break;
		}
		
		testlock.unlock();
		testlock.lock();
	}
}


void run_test()
{
	Threads::sched.new_thread(threada, NULL, "thread a")->wake();
}
